@@ -254,7 +254,7 @@ dec2sep(char *str2, char *str, char sep)
 }
 
 static void
-print_results(int ctxid, uint32_t *eff_timeout)
+print_results(int ctxid, uint64_t *eff_timeout)
 {
 	unsigned int i, j, cnt, ovfl_event;
 	uint64_t value, tot_runs = 0, set_runs;
@@ -283,9 +283,8 @@ print_results(int ctxid, uint32_t *eff_timeout)
 	 * is usally at page size (16KB)
 	 */
 	ret = pfm_read_pmds(ctxid, all_pmds, num_pmds);
-	if (ret == -1) {
+	if (ret == -1)
 		fatal_error("cannot read pmds: %s\n", strerror(errno));
-	}
 
 	/*
 	 * extract all set information
@@ -295,9 +294,8 @@ print_results(int ctxid, uint32_t *eff_timeout)
 	 * is usually at page size (16KB)
 	 */
 	ret = pfm_getinfo_evtsets(ctxid, all_setinfos, num_sets);
-	if (ret == -1) {
+	if (ret == -1)
 		fatal_error("cannot get set info: %s\n", strerror(errno));
-	}
 
 	/*
 	 * compute average number of runs
@@ -320,15 +318,15 @@ print_results(int ctxid, uint32_t *eff_timeout)
 	 *
 	 */
 	if (options.opt_no_header == 0) {
-		printf("# %u Hz period = %u usecs\n# %"PRIu64" cycles @ %lu MHz\n", 
+		printf("# %u Hz period = %u nsecs\n# %"PRIu64" cycles @ %lu MHz\n", 
 			options.smpl_freq, 
-			1000000 / options.smpl_freq, 
+			1000000000 / options.smpl_freq, 
 			options.smpl_period,
 			options.cpu_mhz);
 
 		if (options.opt_ovfl_switch == 0)
 			printf("# using time-based multiplexing\n"
-				"# %u usecs effective switch timeout\n", 
+				"# %"PRIu64" nsecs effective switch timeout\n", 
 				*eff_timeout);
 		else
 			printf("# using overflow-based multiplexing\n");
@@ -408,7 +406,7 @@ measure_one_task(char **argv)
 	pfarg_pmc_t *my_pmcs;
 	pfarg_pmd_t *my_pmds;
 	pfarg_load_t load_arg;
-	uint32_t eff_timeout;
+	uint64_t eff_timeout;
 	pfm_msg_t msg;
 	pid_t pid;
 	int status, ret;
@@ -432,17 +430,13 @@ measure_one_task(char **argv)
 	/*
 	 * create the context
 	 */
-	if (pfm_create_context(ctx, NULL, 0) == -1 ) {
+	ctxid = pfm_create_context(ctx, NULL, NULL, 0);
+	if (ctxid == -1 ) {
 		if (errno == ENOSYS) {
 			fatal_error("Your kernel does not have performance monitoring support!\n");
 		}
 		fatal_error("Can't create PFM context %s\n", strerror(errno));
 	}
-	/*
-	 * extract context id
-	 */
-	ctxid = ctx[0].ctx_fd;
-
 	/*
 	 * set close-on-exec to ensure we will be getting the PFM_END_MSG, i.e.,
 	 * fd not visible to child.
@@ -457,23 +451,22 @@ measure_one_task(char **argv)
 	 * reason. However to avoid special casing set0 for creation, a PFM_CREATE_EVTSETS
 	 * for set0 does not complain and behaves as a PFM_CHANGE_EVTSETS
 	 */
-	vbprintf("requested timeout %u usecs\n", my_sets[0].set_timeout);
+	vbprintf("requested timeout %"PRIu64" nsecs\n", my_sets[0].set_timeout);
 
-	if (pfm_create_evtsets(ctxid, my_sets, num_sets)) {
+	if (pfm_create_evtsets(ctxid, my_sets, num_sets))
 		fatal_error("cannot create sets\n");
-	}
+
 	eff_timeout = my_sets[0].set_timeout;
 
-	vbprintf("effective timeout %u usecs\n", my_sets[0].set_timeout);
+	vbprintf("effective timeout %"PRIu64" nsecs\n", my_sets[0].set_timeout);
 	/*
 	 * Now program the all the registers in one call
 	 *
 	 * Note that there is a limitation on the size of the argument vector
 	 * that can be passed. It is usually set to a page size (16KB).
 	 */
-	if (pfm_write_pmcs(ctxid, my_pmcs, num_pmcs) == -1) {
+	if (pfm_write_pmcs(ctxid, my_pmcs, num_pmcs) == -1)
 		fatal_error("pfm_write_pmcs error errno %d\n",errno);
-	}
 
 	/*
 	 * initialize the PMD registers.
@@ -481,9 +474,8 @@ measure_one_task(char **argv)
 	 * To be read, each PMD must be either written or declared
 	 * as being part of a sample (reg_smpl_pmds)
 	 */
-	if (pfm_write_pmds(ctxid, my_pmds, num_pmds) == -1) {
+	if (pfm_write_pmds(ctxid, my_pmds, num_pmds) == -1)
 		fatal_error("pfm_write_pmds error errno %d\n",errno);
-	}
 
 	/*
 	 * now launch the child code
@@ -509,16 +501,14 @@ measure_one_task(char **argv)
 	 * now attach the context
 	 */
 	load_arg.load_pid = pid;
-	if (pfm_load_context(ctxid, &load_arg) == -1) {
+	if (pfm_load_context(ctxid, &load_arg) == -1)
 		fatal_error("pfm_load_context error errno %d\n",errno);
-	}
 
 	/*
 	 * start monitoring
 	 */
-	if (pfm_start(ctxid, NULL) == -1) {
+	if (pfm_start(ctxid, NULL) == -1)
 		fatal_error("pfm_start error errno %d\n",errno);
-	}
 
 	ptrace(PTRACE_DETACH, pid, NULL, 0);
 	vbprintf("child restarted\n");
@@ -542,9 +532,9 @@ measure_one_task(char **argv)
 	 * mainloop
 	 */
 	ret = read(ctxid, &msg, sizeof(msg));
-	if (ret < sizeof(msg)) {
+	if (ret < sizeof(msg))
 		fatal_error("interrupted read\n");
-	}
+
 	switch(msg.type) {
 		case PFM_MSG_OVFL:
 			fatal_error("unexpected ovfl message\n");
@@ -574,7 +564,8 @@ finish_line:
 		ptrace(PTRACE_DETACH, pid, NULL, 0);
 	}
 
-	if (time_to_quit < 2) print_results(ctxid, &eff_timeout);
+	if (time_to_quit < 2)
+		print_results(ctxid, &eff_timeout);
 
 	close(ctxid);
 
@@ -620,17 +611,13 @@ measure_one_cpu(char **argv)
 	/*
 	 * create the context
 	 */
-	if (pfm_create_context(ctx, NULL, 0) == -1 ) {
+	ctxid = pfm_create_context(ctx, NULL, NULL, 0);
+	if (ctxid == -1) {
 		if (errno == ENOSYS) {
 			fatal_error("Your kernel does not have performance monitoring support!\n");
 		}
 		fatal_error("Can't create PFM context %s\n", strerror(errno));
 	}
-	/*
-	 * extract context id
-	 */
-	ctxid = ctx[0].ctx_fd;
-
 	/*
 	 * set close-on-exec to ensure we will be getting the PFM_END_MSG, i.e.,
 	 * fd not visible to child.
@@ -645,9 +632,8 @@ measure_one_cpu(char **argv)
 	 * reason. However to avoid special casing set0 for creation, a PFM_CREATE_EVTSETS
 	 * for set0 does not complain and behaves as a PFM_CHANGE_EVTSETS
 	 */
-	if (pfm_create_evtsets(ctxid, my_sets, num_sets)) {
+	if (pfm_create_evtsets(ctxid, my_sets, num_sets))
 		fatal_error("cannot create sets\n");
-	}
 
 	/*
 	 * Now program the all the registers in one call
@@ -655,9 +641,8 @@ measure_one_cpu(char **argv)
 	 * Note that there is a limitation on the size of the argument vector
 	 * that can be passed. It is usually set to a page size (16KB).
 	 */
-	if (pfm_write_pmcs(ctxid, my_pmcs, num_pmcs) == -1) {
+	if (pfm_write_pmcs(ctxid, my_pmcs, num_pmcs) == -1)
 		fatal_error("pfm_write_pmcs error errno %d\n",errno);
-	}
 
 	/*
 	 * initialize the PMD registers.
@@ -665,9 +650,8 @@ measure_one_cpu(char **argv)
 	 * To be read, each PMD must be either written or declared
 	 * as being part of a sample (reg_smpl_pmds)
 	 */
-	if (pfm_write_pmds(ctxid, my_pmds, num_pmds) == -1) {
+	if (pfm_write_pmds(ctxid, my_pmds, num_pmds) == -1)
 		fatal_error("pfm_write_pmds error errno %d\n",errno);
-	}
 
 	/*
 	 * now launch the child code
@@ -695,16 +679,14 @@ measure_one_cpu(char **argv)
 	 * now attach the context
 	 */
 	load_arg.load_pid = options.opt_is_system ? getpid() : pid;
-	if (pfm_load_context(ctxid, &load_arg) == -1) {
+	if (pfm_load_context(ctxid, &load_arg) == -1)
 		fatal_error("pfm_load_context error errno %d\n",errno);
-	}
 
 	/*
 	 * start monitoring
 	 */
-	if (pfm_start(ctxid, NULL) == -1) {
+	if (pfm_start(ctxid, NULL) == -1)
 		fatal_error("pfm_start error errno %d\n",errno);
-	}
 
 	if (pid) ptrace(PTRACE_DETACH, pid, NULL, 0);
 
@@ -820,8 +802,9 @@ mainloop(char **argv)
 			p = strchr(str, ',');
 			if (p)
 				*p = '\0';
-			if (pfm_find_full_event(str, &inp.pfp_events[j]) != PFMLIB_SUCCESS)
-				fatal_error("Cannot find %s event for set %d event %d\n", str, i, j);
+			ret = pfm_find_full_event(str, &inp.pfp_events[j]);
+			if (ret != PFMLIB_SUCCESS)
+				fatal_error("event %s for set %d event %d: %s\n", str, i, j, pfm_strerror(ret));
 			if (p)
 				str = p + 1;
 		}
@@ -853,9 +836,8 @@ mainloop(char **argv)
 		/*
 		 * let the library do the hard work
 		 */
-		if ((ret=pfm_dispatch_events(&inp, NULL, &outp, NULL)) != PFMLIB_SUCCESS) {
+		if ((ret=pfm_dispatch_events(&inp, NULL, &outp, NULL)) != PFMLIB_SUCCESS)
 			fatal_error("cannot configure events for set %d: %s\n", i, pfm_strerror(ret));
-		}
 
 		/*
 		 * propagate from libpfm to kernel data structures
@@ -885,7 +867,7 @@ mainloop(char **argv)
 			 * the first overflow of our trigger counter does
 			 * trigger a switch.
 			 */
-			all_pmds[num_pmcs-1].reg_ovfl_switch_cnt = 1;
+			all_pmds[num_pmds-1].reg_ovfl_switch_cnt = 1;
 
 			/*
 			 * We do this even in system-wide mode to ensure
@@ -898,14 +880,14 @@ mainloop(char **argv)
 			all_pmds[num_pmds-1].reg_long_reset  = - options.smpl_period;
 		} else {
 			/*
-			 * setup the switch timeout
+			 * setup the switch timeout (in nanoseconds)
 			 * Note that the actual timeout may be bigger than requested
 			 * due to timer tick granularity. It is always advised to
 			 * check the set_timeout value upon return from set creation.
 			 * The structure will by then contain the actual timeout.
 			 */
 			all_sets[i].set_flags    = PFM_SETFL_TIME_SWITCH;
-			all_sets[i].set_timeout  = 1000000 / options.smpl_freq;
+			all_sets[i].set_timeout  = 1000000000 / options.smpl_freq;
 		}
 #ifdef __ia64__
 		if (options.opt_excl_intr && options.opt_is_system)
@@ -970,9 +952,9 @@ generate_default_sets(void)
 
 	for (i=0; i < 2; i++) {
 		name = malloc(len+1);
-		if (name == NULL) {
+		if (name == NULL)
 			fatal_error("cannot allocate space for event name\n");
-		}
+
 		pfm_get_full_event_name(events+i, name, len+1);
 
 		es = (event_set_t *)malloc(sizeof(event_set_t));
