@@ -40,18 +40,19 @@
  *        - options: --system
  *                   --no-overflow-msg
  *                   --block-on-notify
- *                   --sampler <sampler_uuid>
+ *                   --sampler <sampler_name>
  *        - <context_id>: specify an integer that you want to associate with
  *                        the new context for use in other commands.
  *
- *   load_context <context_id> <event_set_id> <program_id>
+ *   load_context <context_id> <event_set_id> <program_id|cpu_id>
  *      Attach the specified context and event-set to the specified program.
  *        - <context_id>: ID that you specified when creating the context.
  *        - <event_set_id>: ID that you specified when creating an event-set
  *                          within the given context. All contexts automatically
  *                          have an event-set with ID of 0.
- *        - <program_id>: ID that you specified when starting a program with the
- *                        run_program command.
+ *        - <program_id|cpu_id>: ID that you specified when starting a program
+ *                               with the run_program command, or the number of
+ *                               the CPU to attach to for system-wide mode.
  *
  *   unload_context <context_id>
  *      Detach the specified context from the program that it's currently
@@ -118,10 +119,8 @@
  *   create_eventset [options] <context_id> <event_set_id>
  *      Create a new event-set for an existing context.
  *        - options: --next-set <next_event_set_id>
- *                   --timeout <microseconds>
+ *                   --timeout <nanoseconds>
  *                   --switch-on-overflow
- *                   --switch-on-timeout
- *                   --explicit-next-set
  *                   --exclude-idle
  *        - <context_id>: ID that you specified when creating the context.
  *        - <event_set_id>: specify an integer that you want to associate with
@@ -212,6 +211,8 @@
 #include <unistd.h>
 #include <errno.h>
 #include <getopt.h>
+#include <inttypes.h>
+#include <sched.h>
 #include <sys/wait.h>
 #include <sys/ptrace.h>
 #include <perfmon/perfmon.h>
@@ -240,6 +241,8 @@ struct command {
 
 struct context {
 	int id;
+	int fd;
+	int cpu;
 	pfarg_ctx_t ctx_arg;
 	pfm_dfl_smpl_arg_t smpl_arg;
 	struct event_set *event_sets;
@@ -396,6 +399,53 @@ static void remove_program(struct program *prog)
 	}
 }
 
+/**
+ * set_affinity
+ *
+ * When loading or unloading a system-wide context, we must pin the pfmsetup
+ * process to that CPU before making the system call. Also, get the current
+ * affinity and return it to the caller so we can change it back later.
+ **/
+static int set_affinity(int cpu, cpu_set_t *old_cpu_set)
+{
+	cpu_set_t new_cpu_set;
+	int rc;
+
+	rc = sched_getaffinity(0, sizeof(*old_cpu_set), old_cpu_set);
+	if (rc) {
+		rc = errno;
+		LOG_ERROR("Can't get current process affinity mask: %d\n", rc);
+		return rc;
+	}
+
+	CPU_ZERO(&new_cpu_set);
+	CPU_SET(cpu, &new_cpu_set);
+	rc = sched_setaffinity(0, sizeof(new_cpu_set), &new_cpu_set);
+	if (rc) {
+		rc = errno;
+		LOG_ERROR("Can't set process affinity to CPU %d: %d\n", cpu, rc);
+		return rc;
+	}
+
+	return 0;
+}
+
+/**
+ * revert_affinity
+ *
+ * Reset the process affinity to the specified mask.
+ **/
+static void revert_affinity(cpu_set_t *old_cpu_set)
+{
+	int rc;
+
+	rc = sched_setaffinity(0, sizeof(*old_cpu_set), old_cpu_set);
+	if (rc) {
+		/* Not a fatal error if we can't reset the affinity. */
+		LOG_INFO("Can't revert process affinity to original value.\n");
+	}
+}
+
 /**
  * create_context
  *
@@ -403,7 +453,7 @@ static void remove_program(struct program *prog)
  * Options: --system
  *          --no-overflow-msg
  *          --block-on-notify
- *          --sampler <sampler_uuid>
+ *          --sampler <sampler_name>
  *
  * Call the pfm_create_context system-call to create a new perfmon context.
  * Add a new entry to the global 'contexts' list.
@@ -413,7 +463,7 @@ static int create_context(int argc, char **argv)
 	pfarg_ctx_t ctx_arg;
 	pfm_dfl_smpl_arg_t smpl_arg;
 	struct context *new_ctx = NULL;
-	char *sampler_uuid = NULL;
+	char *sampler_name = NULL;
 	void *smpl_p;
 	int no_overflow_msg = FALSE;
 	int block_on_notify = FALSE;
@@ -439,7 +489,7 @@ static int create_context(int argc, char **argv)
 				     long_opts, NULL)) != EOF) {
 		switch (c) {
 		case 1:
-			sampler_uuid = optarg;
+			sampler_name = optarg;
 			break;
 		case 2:
 			system_wide = TRUE;
@@ -482,27 +532,7 @@ static int create_context(int argc, char **argv)
 		goto error;
 	}
 
-	/* Set up the parameters for the system call. */
-	if (sampler_uuid) {
-		sscanf(sampler_uuid, "%02x-%02x-%02x-%02x-%02x-%02x-%02x-%02x"
-				"-%02x-%02x-%02x-%02x-%02x-%02x-%02x-%02x",
-				(unsigned int *)ctx_arg.ctx_smpl_buf_id,
-				(unsigned int *)ctx_arg.ctx_smpl_buf_id+1,
-				(unsigned int *)ctx_arg.ctx_smpl_buf_id+2,
-				(unsigned int *)ctx_arg.ctx_smpl_buf_id+3,
-				(unsigned int *)ctx_arg.ctx_smpl_buf_id+4,
-				(unsigned int *)ctx_arg.ctx_smpl_buf_id+5,
-				(unsigned int *)ctx_arg.ctx_smpl_buf_id+6,
-				(unsigned int *)ctx_arg.ctx_smpl_buf_id+7,
-				(unsigned int *)ctx_arg.ctx_smpl_buf_id+8,
-				(unsigned int *)ctx_arg.ctx_smpl_buf_id+9,
-				(unsigned int *)ctx_arg.ctx_smpl_buf_id+10,
-				(unsigned int *)ctx_arg.ctx_smpl_buf_id+11,
-				(unsigned int *)ctx_arg.ctx_smpl_buf_id+12,
-				(unsigned int *)ctx_arg.ctx_smpl_buf_id+13,
-				(unsigned int *)ctx_arg.ctx_smpl_buf_id+14,
-				(unsigned int *)ctx_arg.ctx_smpl_buf_id+15);
-
+	if (sampler_name) {
 		smpl_arg.buf_size = getpagesize();
 		smpl_p = &smpl_arg;
 		sz = sizeof(smpl_arg);
@@ -516,8 +546,8 @@ static int create_context(int argc, char **argv)
 			    (block_on_notify ? PFM_FL_NOTIFY_BLOCK : 0) |
 			    (remap_eventsets ? PFM_FL_MAP_SETS     : 0);
 
-	rc = pfm_create_context(&ctx_arg, smpl_p, sz);
-	if (rc) {
+	rc = pfm_create_context(&ctx_arg, sampler_name, smpl_p, sz);
+	if (rc == -1) {
 		rc = errno;
 		LOG_ERROR("pfm_create_context system call returned "
 			  "an error: %d.", rc);
@@ -545,32 +575,31 @@ static int create_context(int argc, char **argv)
 	}
 
 	new_ctx->id = ctx_id;
+	new_ctx->fd = rc;
+	new_ctx->cpu = -1;
 	new_ctx->ctx_arg = ctx_arg;
 	new_ctx->smpl_arg = smpl_arg;
 
 	insert_context(new_ctx);
 
 	LOG_INFO("Created context %d with file-descriptor %d.",
-		 new_ctx->id, new_ctx->ctx_arg.ctx_fd);
+		 new_ctx->id, new_ctx->fd);
 
 	return 0;
 
 error:
 	if (new_ctx) {
+		close(new_ctx->fd);
 		free(new_ctx->event_sets);
 		free(new_ctx);
 	}
-	if (ctx_arg.ctx_fd > 0) {
-		close(ctx_arg.ctx_fd);
-	}
-
 	return rc;
 }
 
 /**
  * load_context
  *
- * Arguments: <context_id> <event_set_id> <program_id>
+ * Arguments: <context_id> <event_set_id> <program_id|cpu_id>
  *
  * Call the pfm_load_context system-call to load a perfmon context into the
  * system's performance monitoring unit.
@@ -581,15 +610,16 @@ static int load_context(int argc, char **argv)
 	struct event_set *evt;
 	struct program *prog;
 	pfarg_load_t load_arg;
+	cpu_set_t old_cpu_set;
 	int ctx_id, event_set_id, program_id;
-	int rc;
+	int system_wide, rc;
 
 	ctx_id = strtoul(argv[1], NULL, 0);
 	event_set_id = strtoul(argv[2], NULL, 0);
 	program_id = strtoul(argv[3], NULL, 0);
 
-	if (ctx_id <= 0 || event_set_id < 0 || program_id <= 0) {
-		LOG_ERROR("context ID, event-set ID, and program ID must "
+	if (ctx_id <= 0 || event_set_id < 0 || program_id < 0) {
+		LOG_ERROR("context ID, event-set ID, and program/CPU ID must "
 			  "be positive integers.");
 		return EINVAL;
 	}
@@ -607,18 +637,33 @@ static int load_context(int argc, char **argv)
 			  event_set_id, ctx_id);
 		return EINVAL;
 	}
+	load_arg.load_set = evt->id;
 
-	prog = find_program(program_id);
-	if (!prog) {
-		LOG_ERROR("Can't find program with ID %d.", program_id);
-		return EINVAL;
+	system_wide = ctx->ctx_arg.ctx_flags & PFM_FL_SYSTEM_WIDE;
+	if (system_wide) {
+		if (ctx->cpu >= 0) {
+			LOG_ERROR("Trying to load context %d which is already "
+				  "loaded on CPU %d.\n", ctx_id, ctx->cpu);
+			return EBUSY;
+		}
+
+		rc = set_affinity(program_id, &old_cpu_set);
+		if (rc) {
+			return rc;
+		}
+
+		/* Specify the CPU as the PID. */
+		load_arg.load_pid = program_id;
+	} else {
+		prog = find_program(program_id);
+		if (!prog) {
+			LOG_ERROR("Can't find program with ID %d.", program_id);
+			return EINVAL;
+		}
+		load_arg.load_pid = prog->pid;
 	}
 
-	/* Set up the parameters for the system call. */
-	load_arg.load_set = evt->id;
-	load_arg.load_pid = prog->pid;
-
-	rc = pfm_load_context(ctx->ctx_arg.ctx_fd, &load_arg);
+	rc = pfm_load_context(ctx->fd, &load_arg);
 	if (rc) {
 		rc = errno;
 		LOG_ERROR("pfm_load_context system call returned "
@@ -626,8 +671,16 @@ static int load_context(int argc, char **argv)
 		return rc;
 	}
 
-	LOG_INFO("Loaded context %d, event-set %d onto program %d.",
-		 ctx_id, event_set_id, program_id);
+	if (system_wide) {
+		/* Keep track of which CPU this context is loaded on. */
+		ctx->cpu = program_id;
+
+		revert_affinity(&old_cpu_set);
+	}
+
+	LOG_INFO("Loaded context %d, event-set %d onto %s %d.",
+		 ctx_id, event_set_id, system_wide ? "cpu" : "program",
+		 program_id);
 
 	return 0;
 }
@@ -643,6 +696,8 @@ static int load_context(int argc, char **argv)
 static int unload_context(int argc, char **argv)
 {
 	struct context *ctx;
+	cpu_set_t old_cpu_set;
+	int system_wide;
 	int ctx_id;
 	int rc;
 
@@ -658,7 +713,22 @@ static int unload_context(int argc, char **argv)
 		return EINVAL;
 	}
 
-	rc = pfm_unload_context(ctx->ctx_arg.ctx_fd);
+	system_wide = ctx->ctx_arg.ctx_flags & PFM_FL_SYSTEM_WIDE;
+	if (system_wide) {
+		if (ctx->cpu < 0) {
+			/* This context isn't loaded on any CPU. */
+			LOG_ERROR("Trying to unload context %d that isn't "
+				  "loaded.\n", ctx_id);
+			return EINVAL;
+		}
+
+		rc = set_affinity(ctx->cpu, &old_cpu_set);
+		if (rc) {
+			return rc;
+		}
+	}
+
+	rc = pfm_unload_context(ctx->fd);
 	if (rc) {
 		rc = errno;
 		LOG_ERROR("pfm_unload_context system call returned "
@@ -666,6 +736,11 @@ static int unload_context(int argc, char **argv)
 		return rc;
 	}
 
+	if (system_wide) {
+		ctx->cpu = -1;
+		revert_affinity(&old_cpu_set);
+	}
+
 	LOG_INFO("Unloaded context %d.", ctx_id);
 
 	return 0;
@@ -700,7 +775,7 @@ static int close_context(int argc, char **argv)
 	/* There's no perfmon system-call to delete a context. We simply call
 	 * close on the file handle.
 	 */
-	close(ctx->ctx_arg.ctx_fd);
+	close(ctx->fd);
 	remove_context(ctx);
 
 	for (evt = ctx->event_sets; evt; evt = next_evt) {
@@ -726,10 +801,11 @@ static int write_pmc(int argc, char **argv)
 	struct context *ctx;
 	struct event_set *evt;
 	pfarg_pmc_t *pmc_args = NULL;
+	cpu_set_t old_cpu_set;
 	int ctx_id, event_set_id;
 	int pmc_id, num_pmcs;
 	unsigned long long pmc_value;
-	int i, rc;
+	int system_wide, i, rc;
 
 	ctx_id = strtoul(argv[1], NULL, 0);
 	event_set_id = strtoul(argv[2], NULL, 0);
@@ -776,7 +852,15 @@ static int write_pmc(int argc, char **argv)
 		pmc_args[i].reg_value = pmc_value;
 	}
 
-	rc = pfm_write_pmcs(ctx->ctx_arg.ctx_fd, pmc_args, num_pmcs);
+	system_wide = ctx->ctx_arg.ctx_flags & PFM_FL_SYSTEM_WIDE;
+	if (system_wide && ctx->cpu >= 0) {
+		rc = set_affinity(ctx->cpu, &old_cpu_set);
+		if (rc) {
+			goto out;
+		}
+	}
+
+	rc = pfm_write_pmcs(ctx->fd, pmc_args, num_pmcs);
 	if (rc) {
 		rc = errno;
 		LOG_ERROR("pfm_write_pmcs system call returned "
@@ -784,6 +868,10 @@ static int write_pmc(int argc, char **argv)
 		goto out;
 	}
 
+	if (system_wide && ctx->cpu >= 0) {
+		revert_affinity(&old_cpu_set);
+	}
+
 	/* Check each PMC for success or failure. */
 	for (i = 0; i < num_pmcs; i++) {
 		if (pmc_args[i].reg_flags & PFM_REG_RETFL_NOTAVAIL) {
@@ -822,10 +910,11 @@ static int write_pmd(int argc, char **argv)
 	struct context *ctx;
 	struct event_set *evt;
 	pfarg_pmd_t *pmd_args = NULL;
+	cpu_set_t old_cpu_set;
 	int ctx_id, event_set_id;
 	int pmd_id, num_pmds;
 	unsigned long long pmd_value;
-	int i, rc;
+	int system_wide, i, rc;
 
 	ctx_id = strtoul(argv[1], NULL, 0);
 	event_set_id = strtoul(argv[2], NULL, 0);
@@ -872,7 +961,15 @@ static int write_pmd(int argc, char **argv)
 		pmd_args[i].reg_value = pmd_value;
 	}
 
-	rc = pfm_write_pmds(ctx->ctx_arg.ctx_fd, pmd_args, num_pmds);
+	system_wide = ctx->ctx_arg.ctx_flags & PFM_FL_SYSTEM_WIDE;
+	if (system_wide && ctx->cpu >= 0) {
+		rc = set_affinity(ctx->cpu, &old_cpu_set);
+		if (rc) {
+			goto out;
+		}
+	}
+
+	rc = pfm_write_pmds(ctx->fd, pmd_args, num_pmds);
 	if (rc) {
 		rc = errno;
 		LOG_ERROR("pfm_write_pmds system call returned "
@@ -880,6 +977,10 @@ static int write_pmd(int argc, char **argv)
 		goto out;
 	}
 
+	if (system_wide && ctx->cpu >= 0) {
+		revert_affinity(&old_cpu_set);
+	}
+
 	/* Check each PMD for success or failure. */
 	for (i = 0; i < num_pmds; i++) {
 		if (pmd_args[i].reg_flags & PFM_REG_RETFL_NOTAVAIL) {
@@ -918,9 +1019,10 @@ static int read_pmd(int argc, char **argv)
 	struct context *ctx;
 	struct event_set *evt;
 	pfarg_pmd_t *pmd_args = NULL;
+	cpu_set_t old_cpu_set;
 	int ctx_id, event_set_id;
 	int pmd_id, num_pmds;
-	int i, rc;
+	int system_wide, i, rc;
 
 	ctx_id = strtoul(argv[1], NULL, 0);
 	event_set_id = strtoul(argv[2], NULL, 0);
@@ -964,7 +1066,15 @@ static int read_pmd(int argc, char **argv)
 		pmd_args[i].reg_set = evt->id;
 	}
 
-	rc = pfm_read_pmds(ctx->ctx_arg.ctx_fd, pmd_args, num_pmds);
+	system_wide = ctx->ctx_arg.ctx_flags & PFM_FL_SYSTEM_WIDE;
+	if (system_wide && ctx->cpu >= 0) {
+		rc = set_affinity(ctx->cpu, &old_cpu_set);
+		if (rc) {
+			goto out;
+		}
+	}
+
+	rc = pfm_read_pmds(ctx->fd, pmd_args, num_pmds);
 	if (rc) {
 		rc = errno;
 		LOG_ERROR("pfm_read_pmds system call returned "
@@ -972,6 +1082,10 @@ static int read_pmd(int argc, char **argv)
 		goto out;
 	}
 
+	if (system_wide && ctx->cpu >= 0) {
+		revert_affinity(&old_cpu_set);
+	}
+
 	/* Check each PMD for success or failure. */
 	for (i = 0; i < num_pmds; i++) {
 		if (pmd_args[i].reg_flags & PFM_REG_RETFL_NOTAVAIL) {
@@ -1011,8 +1125,9 @@ static int start_counting(int argc, char **argv)
 	pfarg_start_t start_arg;
 	struct context *ctx;
 	struct event_set *evt;
+	cpu_set_t old_cpu_set;
 	int ctx_id, event_set_id;
-	int rc;
+	int system_wide, rc;
 
 	memset(&start_arg, 0, sizeof(start_arg));
 
@@ -1040,13 +1155,25 @@ static int start_counting(int argc, char **argv)
 
 	start_arg.start_set = evt->id;
 
-	rc = pfm_start(ctx->ctx_arg.ctx_fd, &start_arg);
+	system_wide = ctx->ctx_arg.ctx_flags & PFM_FL_SYSTEM_WIDE;
+	if (system_wide && ctx->cpu >= 0) {
+		rc = set_affinity(ctx->cpu, &old_cpu_set);
+		if (rc) {
+			return rc;
+		}
+	}
+
+	rc = pfm_start(ctx->fd, &start_arg);
 	if (rc) {
 		rc = errno;
 		LOG_ERROR("pfm_start system call returned an error: %d.", rc);
 		return rc;
 	}
 
+	if (system_wide && ctx->cpu >= 0) {
+		revert_affinity(&old_cpu_set);
+	}
+
 	LOG_INFO("Started counting for context %d, event-set %d.",
 		 ctx_id, event_set_id);
 
@@ -1064,6 +1191,8 @@ static int start_counting(int argc, char **argv)
 static int stop_counting(int argc, char **argv)
 {
 	struct context *ctx;
+	cpu_set_t old_cpu_set;
+	int system_wide;
 	int ctx_id;
 	int rc;
 
@@ -1080,13 +1209,25 @@ static int stop_counting(int argc, char **argv)
 		return EINVAL;
 	}
 
-	rc = pfm_stop(ctx->ctx_arg.ctx_fd);
+	system_wide = ctx->ctx_arg.ctx_flags & PFM_FL_SYSTEM_WIDE;
+	if (system_wide && ctx->cpu >= 0) {
+		rc = set_affinity(ctx->cpu, &old_cpu_set);
+		if (rc) {
+			return rc;
+		}
+	}
+
+	rc = pfm_stop(ctx->fd);
 	if (rc) {
 		rc = errno;
 		LOG_ERROR("pfm_stop system call returned an error: %d.", rc);
 		return rc;
 	}
 
+	if (system_wide && ctx->cpu >= 0) {
+		revert_affinity(&old_cpu_set);
+	}
+
 	LOG_INFO("Stopped counting for context %d.", ctx_id);
 
 	return 0;
@@ -1103,6 +1244,8 @@ static int stop_counting(int argc, char **argv)
 static int restart_counting(int argc, char **argv)
 {
 	struct context *ctx;
+	cpu_set_t old_cpu_set;
+	int system_wide;
 	int ctx_id;
 	int rc;
 
@@ -1119,13 +1262,25 @@ static int restart_counting(int argc, char **argv)
 		return EINVAL;
 	}
 
-	rc = pfm_restart(ctx->ctx_arg.ctx_fd);
+	system_wide = ctx->ctx_arg.ctx_flags & PFM_FL_SYSTEM_WIDE;
+	if (system_wide && ctx->cpu >= 0) {
+		rc = set_affinity(ctx->cpu, &old_cpu_set);
+		if (rc) {
+			return rc;
+		}
+	}
+
+	rc = pfm_restart(ctx->fd);
 	if (rc) {
 		rc = errno;
 		LOG_ERROR("pfm_restart system call returned an error: %d.", rc);
 		return rc;
 	}
 
+	if (system_wide && ctx->cpu >= 0) {
+		revert_affinity(&old_cpu_set);
+	}
+
 	LOG_INFO("Restarted counting for context %d.", ctx_id);
 
 	return 0;
@@ -1136,10 +1291,8 @@ static int restart_counting(int argc, char **argv)
  *
  * Arguments: [options] <context_id> <event_set_id>
  * Options: --next-set <next_event_set_id>
- *          --timeout <microseconds>
+ *          --timeout <nanoseconds>
  *          --switch-on-overflow
- *          --switch-on-timeout
- *          --explicit-next-set
  *          --exclude-idle
  **/
 static int create_eventset(int argc, char **argv)
@@ -1147,25 +1300,26 @@ static int create_eventset(int argc, char **argv)
 	pfarg_setdesc_t set_arg;
 	struct context *ctx;
 	struct event_set *evt;
+	cpu_set_t old_cpu_set;
 	int ctx_id, event_set_id, next_set_id = 0;
 	unsigned long timeout = 0;
 	int switch_on_overflow = FALSE;
 	int switch_on_timeout = FALSE;
 	int explicit_next_set = FALSE;
 	int exclude_idle = FALSE;
-	int c, rc;
+	int new_set = FALSE;
+	int system_wide,c, rc;
 	struct option long_opts[] = {
 		{"next-set",           required_argument, NULL, 1},
 		{"timeout",            required_argument, NULL, 2},
 		{"switch-on-overflow", no_argument,       NULL, 3},
-		{"switch-on-timeout",  no_argument,       NULL, 4},
-		{"explicit-next-set",  no_argument,       NULL, 5},
-		{"exclude-idle",       no_argument,       NULL, 6},
+		{"exclude-idle",       no_argument,       NULL, 4},
 		{NULL,                 0,                 NULL, 0} };
 
 	memset(&set_arg, 0, sizeof(set_arg));
 
 	opterr = 0;
+	optind = 0;
 	while ((c = getopt_long_only(argc, argv, "",
 				     long_opts, NULL)) != EOF) {
 		switch (c) {
@@ -1176,6 +1330,7 @@ static int create_eventset(int argc, char **argv)
 					  "positive integer.");
 				return EINVAL;
 			}
+			explicit_next_set = TRUE;
 			break;
 		case 2:
 			timeout = strtoul(optarg, NULL, 0);
@@ -1184,17 +1339,12 @@ static int create_eventset(int argc, char **argv)
 					  "non-zero integer.");
 				return EINVAL;
 			}
+			switch_on_timeout = TRUE;
 			break;
 		case 3:
 			switch_on_overflow = TRUE;
 			break;
 		case 4:
-			switch_on_timeout = TRUE;
-			break;
-		case 5:
-			explicit_next_set = TRUE;
-			break;
-		case 6:
 			exclude_idle = TRUE;
 			break;
 		default:
@@ -1203,13 +1353,13 @@ static int create_eventset(int argc, char **argv)
 		}
 	}
 
-	if (argc < optind + 3) {
+	if (argc < optind + 2) {
 		USAGE("create_eventset [options] <context_id> <event_set_id>");
 		return EINVAL;
 	}
 
-	ctx_id = strtoul(argv[1], NULL, 0);
-	event_set_id = strtoul(argv[2], NULL, 0);
+	ctx_id = strtoul(argv[optind], NULL, 0);
+	event_set_id = strtoul(argv[optind+1], NULL, 0);
 
 	if (ctx_id <= 0 || event_set_id < 0) {
 		LOG_ERROR("context ID and event-set ID must be "
@@ -1223,28 +1373,41 @@ static int create_eventset(int argc, char **argv)
 		return EINVAL;
 	}
 
-	evt = find_event_set(ctx, event_set_id);
-	if (evt) {
-		LOG_ERROR("Event-set with ID %d already exists in context %d.",
-			  event_set_id, ctx_id);
+	if (switch_on_timeout && switch_on_overflow) {
+		LOG_ERROR("Cannot switch set %d (context %d) on both "
+			  "timeout and overflow.", event_set_id, ctx_id);
 		return EINVAL;
 	}
 
-	evt = calloc(1, sizeof(*evt));
+	evt = find_event_set(ctx, event_set_id);
 	if (!evt) {
-		LOG_ERROR("Can't allocate structure for new event-set %d in "
-			  "context %d.", event_set_id, ctx_id);
-		return ENOMEM;
+		evt = calloc(1, sizeof(*evt));
+		if (!evt) {
+			LOG_ERROR("Can't allocate structure for new event-set "
+				  "%d in context %d.", event_set_id, ctx_id);
+			return ENOMEM;
+		}
+		evt->id = event_set_id;
+		new_set = TRUE;
 	}
 
 	set_arg.set_id = event_set_id;
 	set_arg.set_id_next = next_set_id;
-	set_arg.set_timeout = timeout;
+	set_arg.set_timeout = timeout; /* in nanseconds */
 	set_arg.set_flags = (switch_on_overflow ? PFM_SETFL_OVFL_SWITCH : 0) |
 			    (switch_on_timeout  ? PFM_SETFL_TIME_SWITCH : 0) |
 			    (explicit_next_set  ? PFM_SETFL_EXPL_NEXT   : 0);
 
-	rc = pfm_create_evtsets(ctx->ctx_arg.ctx_fd, &set_arg, 1);
+	system_wide = ctx->ctx_arg.ctx_flags & PFM_FL_SYSTEM_WIDE;
+	if (system_wide && ctx->cpu >= 0) {
+		rc = set_affinity(ctx->cpu, &old_cpu_set);
+		if (rc) {
+			free(evt);
+			return rc;
+		}
+	}
+
+	rc = pfm_create_evtsets(ctx->fd, &set_arg, 1);
 	if (rc) {
 		rc = errno;
 		LOG_ERROR("pfm_create_evtsets system call returned "
@@ -1253,10 +1416,20 @@ static int create_eventset(int argc, char **argv)
 		return rc;
 	}
 
-	evt->id = event_set_id;
-	insert_event_set(ctx, evt);
+	if (system_wide && ctx->cpu >= 0) {
+		revert_affinity(&old_cpu_set);
+	}
 
-	LOG_INFO("Created event-set %d in context %d.", event_set_id, ctx_id);
+	if (new_set) {
+		insert_event_set(ctx, evt);
+	}
+
+	LOG_INFO("%s event-set %d in context %d.",
+		 new_set ? "Created" : "Modified", event_set_id, ctx_id);
+	if (switch_on_timeout) {
+		LOG_INFO("   Actual timeout set to %llu ns.",
+			 (unsigned long long)set_arg.set_timeout);
+	}
 
 	return 0;
 }
@@ -1271,8 +1444,9 @@ static int delete_eventset(int argc, char **argv)
 	pfarg_setdesc_t set_arg;
 	struct context *ctx;
 	struct event_set *evt;
+	cpu_set_t old_cpu_set;
 	int ctx_id, event_set_id;
-	int rc;
+	int system_wide, rc;
 
 	memset(&set_arg, 0, sizeof(set_arg));
 
@@ -1300,7 +1474,15 @@ static int delete_eventset(int argc, char **argv)
 
 	set_arg.set_id = evt->id;
 
-	rc = pfm_delete_evtsets(ctx->ctx_arg.ctx_fd, &set_arg, 1);
+	system_wide = ctx->ctx_arg.ctx_flags & PFM_FL_SYSTEM_WIDE;
+	if (system_wide && ctx->cpu >= 0) {
+		rc = set_affinity(ctx->cpu, &old_cpu_set);
+		if (rc) {
+			return rc;
+		}
+	}
+
+	rc = pfm_delete_evtsets(ctx->fd, &set_arg, 1);
 	if (rc) {
 		rc = errno;
 		LOG_ERROR("pfm_delete_evtsets system call returned "
@@ -1308,6 +1490,10 @@ static int delete_eventset(int argc, char **argv)
 		return rc;
 	}
 
+	if (system_wide && ctx->cpu >= 0) {
+		revert_affinity(&old_cpu_set);
+	}
+
 	remove_event_set(ctx, evt);
 	free(evt);
 
@@ -1326,8 +1512,9 @@ static int getinfo_eventset(int argc, char **argv)
 	pfarg_setinfo_t set_arg;
 	struct context *ctx;
 	struct event_set *evt;
+	cpu_set_t old_cpu_set;
 	int ctx_id, event_set_id;
-	int rc;
+	int system_wide, rc;
 
 	memset(&set_arg, 0, sizeof(set_arg));
 
@@ -1355,7 +1542,15 @@ static int getinfo_eventset(int argc, char **argv)
 
 	set_arg.set_id = evt->id;
 
-	rc = pfm_getinfo_evtsets(ctx->ctx_arg.ctx_fd, &set_arg, 1);
+	system_wide = ctx->ctx_arg.ctx_flags & PFM_FL_SYSTEM_WIDE;
+	if (system_wide && ctx->cpu >= 0) {
+		rc = set_affinity(ctx->cpu, &old_cpu_set);
+		if (rc) {
+			return rc;
+		}
+	}
+
+	rc = pfm_getinfo_evtsets(ctx->fd, &set_arg, 1);
 	if (rc) {
 		rc = errno;
 		LOG_ERROR("pfm_getinfo_evtsets system call returned "
@@ -1363,11 +1558,15 @@ static int getinfo_eventset(int argc, char **argv)
 		return rc;
 	}
 
+	if (system_wide && ctx->cpu >= 0) {
+		revert_affinity(&old_cpu_set);
+	}
+
 	LOG_INFO("Got info for event-set %d in context %d.", event_set_id, ctx_id);
 	LOG_INFO("   Next set: %u", set_arg.set_id_next);
 	LOG_INFO("   Flags: 0x%x", set_arg.set_flags);
 	LOG_INFO("   Runs: %llu", (unsigned long long)set_arg.set_runs);
-	LOG_INFO("   Timeout: %u", set_arg.set_timeout);
+	LOG_INFO("   Timeout: %"PRIu64, set_arg.set_timeout);
 	LOG_INFO("   MMAP offset: %llu", (unsigned long long)set_arg.set_mmap_offset);
 
 	return 0;
@@ -1560,9 +1759,12 @@ static int _sleep(int argc, char **argv)
 		return EINVAL;
 	}
 
-	sleep(seconds);
+	LOG_INFO("Sleeping for %d seconds.", seconds);
 
-	LOG_INFO("Slept for %d seconds.", seconds);
+	while (seconds > 0)
+		seconds = sleep(seconds);
+
+	LOG_INFO("Done sleeping.");
 
 	return 0;
 }
@@ -1581,11 +1783,11 @@ static struct command _commands[] = {
 
 	{ "create_context", "cc",
 	  "<context_id> [--system] [--no-overflow-msg] "
-	    "[--block-on-notify] [--sampler <sampler_uuid>]",
+	    "[--block-on-notify] [--sampler <sampler_name>]",
 	  create_context, 1 },
 
 	{ "load_context", "load",
-	  "<context_id> <event_set_id> <program_id>",
+	  "<context_id> <event_set_id> <program_id|cpu_id>",
 	  load_context, 3 },
 
 	{ "unload_context", "unload",
@@ -1622,8 +1824,7 @@ static struct command _commands[] = {
 
 	{ "create_eventset", "ce",
 	  "<context_id> <event_set_id> [--next-set <next_event_set_id>] "
-	    "[--timeout <microseconds>] [--switch-on-overflow] "
-	    "[--switch-on-timeout] [--explicit-next-set] [--exclude-idle]",
+	    "[--timeout <nanoseconds>] [--switch-on-overflow] [--exclude-idle]",
 	  create_eventset, 2 },
 
 
